"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from kombo.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from kombo.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Any, Dict, List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


GetAtsApplicationsPositiveResponseOutcome = Literal[
    "PENDING",
    "HIRED",
    "DECLINED",
]
r"""Parsed status of the application. If Kombo identifies that the application was accepted and the candidate hired, it will be `HIRED`. If the application was rejected or the candidate declined, it will be `DECLINED`. If the application is still in process, it will be `PENDING`.
Kombo will always try to deliver this information as reliably as possible.
"""


class GetAtsApplicationsPositiveResponseAnswer7TypedDict(TypedDict):
    raw: NotRequired[Nullable[Any]]
    r"""We pass the original question data along so you can handle it."""


class GetAtsApplicationsPositiveResponseAnswer7(BaseModel):
    raw: OptionalNullable[Any] = UNSET
    r"""We pass the original question data along so you can handle it."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["raw"]
        nullable_fields = ["raw"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class GetAtsApplicationsPositiveResponseQuestionUnknownTypedDict(TypedDict):
    remote_id: Nullable[str]
    title: str
    type: Literal["UNKNOWN"]
    r"""When we're not able to map a specific question type yet, we will return this type. Every `UNKNOWN` question will also be parsed and unified by us at some point."""


class GetAtsApplicationsPositiveResponseQuestionUnknown(BaseModel):
    remote_id: Nullable[str]

    title: str

    TYPE: Annotated[
        Annotated[Literal["UNKNOWN"], AfterValidator(validate_const("UNKNOWN"))],
        pydantic.Field(alias="type"),
    ] = "UNKNOWN"
    r"""When we're not able to map a specific question type yet, we will return this type. Every `UNKNOWN` question will also be parsed and unified by us at some point."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["remote_id"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class GetAtsApplicationsPositiveResponseScreeningQuestionAnswer7TypedDict(TypedDict):
    answer: GetAtsApplicationsPositiveResponseAnswer7TypedDict
    question: GetAtsApplicationsPositiveResponseQuestionUnknownTypedDict


class GetAtsApplicationsPositiveResponseScreeningQuestionAnswer7(BaseModel):
    answer: GetAtsApplicationsPositiveResponseAnswer7

    question: GetAtsApplicationsPositiveResponseQuestionUnknown


class GetAtsApplicationsPositiveResponseAnswer6TypedDict(TypedDict):
    date_: Nullable[datetime]


class GetAtsApplicationsPositiveResponseAnswer6(BaseModel):
    date_: Annotated[Nullable[datetime], pydantic.Field(alias="date")]

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["date"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class GetAtsApplicationsPositiveResponseQuestionDateTypedDict(TypedDict):
    remote_id: Nullable[str]
    title: str
    type: Literal["DATE"]


class GetAtsApplicationsPositiveResponseQuestionDate(BaseModel):
    remote_id: Nullable[str]

    title: str

    TYPE: Annotated[
        Annotated[Literal["DATE"], AfterValidator(validate_const("DATE"))],
        pydantic.Field(alias="type"),
    ] = "DATE"

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["remote_id"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class GetAtsApplicationsPositiveResponseScreeningQuestionAnswer6TypedDict(TypedDict):
    answer: GetAtsApplicationsPositiveResponseAnswer6TypedDict
    question: GetAtsApplicationsPositiveResponseQuestionDateTypedDict


class GetAtsApplicationsPositiveResponseScreeningQuestionAnswer6(BaseModel):
    answer: GetAtsApplicationsPositiveResponseAnswer6

    question: GetAtsApplicationsPositiveResponseQuestionDate


class GetAtsApplicationsPositiveResponseAnswer5TypedDict(TypedDict):
    number: Nullable[float]


class GetAtsApplicationsPositiveResponseAnswer5(BaseModel):
    number: Nullable[float]

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["number"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class GetAtsApplicationsPositiveResponseQuestionNumberTypedDict(TypedDict):
    remote_id: Nullable[str]
    title: str
    type: Literal["NUMBER"]


class GetAtsApplicationsPositiveResponseQuestionNumber(BaseModel):
    remote_id: Nullable[str]

    title: str

    TYPE: Annotated[
        Annotated[Literal["NUMBER"], AfterValidator(validate_const("NUMBER"))],
        pydantic.Field(alias="type"),
    ] = "NUMBER"

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["remote_id"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class GetAtsApplicationsPositiveResponseScreeningQuestionAnswer5TypedDict(TypedDict):
    answer: GetAtsApplicationsPositiveResponseAnswer5TypedDict
    question: GetAtsApplicationsPositiveResponseQuestionNumberTypedDict


class GetAtsApplicationsPositiveResponseScreeningQuestionAnswer5(BaseModel):
    answer: GetAtsApplicationsPositiveResponseAnswer5

    question: GetAtsApplicationsPositiveResponseQuestionNumber


class GetAtsApplicationsPositiveResponseAnswer4TypedDict(TypedDict):
    checked: Nullable[bool]


class GetAtsApplicationsPositiveResponseAnswer4(BaseModel):
    checked: Nullable[bool]

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["checked"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class GetAtsApplicationsPositiveResponseQuestionBooleanTypedDict(TypedDict):
    remote_id: Nullable[str]
    title: str
    type: Literal["BOOLEAN"]


class GetAtsApplicationsPositiveResponseQuestionBoolean(BaseModel):
    remote_id: Nullable[str]

    title: str

    TYPE: Annotated[
        Annotated[Literal["BOOLEAN"], AfterValidator(validate_const("BOOLEAN"))],
        pydantic.Field(alias="type"),
    ] = "BOOLEAN"

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["remote_id"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class GetAtsApplicationsPositiveResponseScreeningQuestionAnswer4TypedDict(TypedDict):
    answer: GetAtsApplicationsPositiveResponseAnswer4TypedDict
    question: GetAtsApplicationsPositiveResponseQuestionBooleanTypedDict


class GetAtsApplicationsPositiveResponseScreeningQuestionAnswer4(BaseModel):
    answer: GetAtsApplicationsPositiveResponseAnswer4

    question: GetAtsApplicationsPositiveResponseQuestionBoolean


class GetAtsApplicationsPositiveResponseAnswer3TypedDict(TypedDict):
    choices: NotRequired[List[str]]


class GetAtsApplicationsPositiveResponseAnswer3(BaseModel):
    choices: Optional[List[str]] = None


class GetAtsApplicationsPositiveResponseQuestionMultiSelectTypedDict(TypedDict):
    remote_id: Nullable[str]
    title: str
    type: Literal["MULTI_SELECT"]


class GetAtsApplicationsPositiveResponseQuestionMultiSelect(BaseModel):
    remote_id: Nullable[str]

    title: str

    TYPE: Annotated[
        Annotated[
            Literal["MULTI_SELECT"], AfterValidator(validate_const("MULTI_SELECT"))
        ],
        pydantic.Field(alias="type"),
    ] = "MULTI_SELECT"

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["remote_id"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class GetAtsApplicationsPositiveResponseScreeningQuestionAnswer3TypedDict(TypedDict):
    answer: GetAtsApplicationsPositiveResponseAnswer3TypedDict
    question: GetAtsApplicationsPositiveResponseQuestionMultiSelectTypedDict


class GetAtsApplicationsPositiveResponseScreeningQuestionAnswer3(BaseModel):
    answer: GetAtsApplicationsPositiveResponseAnswer3

    question: GetAtsApplicationsPositiveResponseQuestionMultiSelect


class GetAtsApplicationsPositiveResponseAnswer2TypedDict(TypedDict):
    choice: Nullable[str]


class GetAtsApplicationsPositiveResponseAnswer2(BaseModel):
    choice: Nullable[str]

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["choice"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class GetAtsApplicationsPositiveResponseQuestionSingleSelectTypedDict(TypedDict):
    remote_id: Nullable[str]
    title: str
    type: Literal["SINGLE_SELECT"]


class GetAtsApplicationsPositiveResponseQuestionSingleSelect(BaseModel):
    remote_id: Nullable[str]

    title: str

    TYPE: Annotated[
        Annotated[
            Literal["SINGLE_SELECT"], AfterValidator(validate_const("SINGLE_SELECT"))
        ],
        pydantic.Field(alias="type"),
    ] = "SINGLE_SELECT"

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["remote_id"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class GetAtsApplicationsPositiveResponseScreeningQuestionAnswer2TypedDict(TypedDict):
    answer: GetAtsApplicationsPositiveResponseAnswer2TypedDict
    question: GetAtsApplicationsPositiveResponseQuestionSingleSelectTypedDict


class GetAtsApplicationsPositiveResponseScreeningQuestionAnswer2(BaseModel):
    answer: GetAtsApplicationsPositiveResponseAnswer2

    question: GetAtsApplicationsPositiveResponseQuestionSingleSelect


class GetAtsApplicationsPositiveResponseAnswer1TypedDict(TypedDict):
    content: Nullable[str]


class GetAtsApplicationsPositiveResponseAnswer1(BaseModel):
    content: Nullable[str]

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["content"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class GetAtsApplicationsPositiveResponseQuestionTextTypedDict(TypedDict):
    remote_id: Nullable[str]
    title: str
    type: Literal["TEXT"]


class GetAtsApplicationsPositiveResponseQuestionText(BaseModel):
    remote_id: Nullable[str]

    title: str

    TYPE: Annotated[
        Annotated[Literal["TEXT"], AfterValidator(validate_const("TEXT"))],
        pydantic.Field(alias="type"),
    ] = "TEXT"

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["remote_id"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class GetAtsApplicationsPositiveResponseScreeningQuestionAnswer1TypedDict(TypedDict):
    answer: GetAtsApplicationsPositiveResponseAnswer1TypedDict
    question: GetAtsApplicationsPositiveResponseQuestionTextTypedDict


class GetAtsApplicationsPositiveResponseScreeningQuestionAnswer1(BaseModel):
    answer: GetAtsApplicationsPositiveResponseAnswer1

    question: GetAtsApplicationsPositiveResponseQuestionText


GetAtsApplicationsPositiveResponseScreeningQuestionAnswerUnionTypedDict = TypeAliasType(
    "GetAtsApplicationsPositiveResponseScreeningQuestionAnswerUnionTypedDict",
    Union[
        GetAtsApplicationsPositiveResponseScreeningQuestionAnswer1TypedDict,
        GetAtsApplicationsPositiveResponseScreeningQuestionAnswer2TypedDict,
        GetAtsApplicationsPositiveResponseScreeningQuestionAnswer3TypedDict,
        GetAtsApplicationsPositiveResponseScreeningQuestionAnswer4TypedDict,
        GetAtsApplicationsPositiveResponseScreeningQuestionAnswer5TypedDict,
        GetAtsApplicationsPositiveResponseScreeningQuestionAnswer6TypedDict,
        GetAtsApplicationsPositiveResponseScreeningQuestionAnswer7TypedDict,
    ],
)


GetAtsApplicationsPositiveResponseScreeningQuestionAnswerUnion = TypeAliasType(
    "GetAtsApplicationsPositiveResponseScreeningQuestionAnswerUnion",
    Union[
        GetAtsApplicationsPositiveResponseScreeningQuestionAnswer1,
        GetAtsApplicationsPositiveResponseScreeningQuestionAnswer2,
        GetAtsApplicationsPositiveResponseScreeningQuestionAnswer3,
        GetAtsApplicationsPositiveResponseScreeningQuestionAnswer4,
        GetAtsApplicationsPositiveResponseScreeningQuestionAnswer5,
        GetAtsApplicationsPositiveResponseScreeningQuestionAnswer6,
        GetAtsApplicationsPositiveResponseScreeningQuestionAnswer7,
    ],
)


class GetAtsApplicationsPositiveResponseEmailAddressTypedDict(TypedDict):
    type: Nullable[str]
    r"""Kombo exposes type information through this field. If we don't get any information from the tool, we will set this to `null`."""
    email_address: NotRequired[Nullable[str]]


class GetAtsApplicationsPositiveResponseEmailAddress(BaseModel):
    type: Nullable[str]
    r"""Kombo exposes type information through this field. If we don't get any information from the tool, we will set this to `null`."""

    email_address: OptionalNullable[str] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["email_address"]
        nullable_fields = ["email_address", "type"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class GetAtsApplicationsPositiveResponsePhoneNumberTypedDict(TypedDict):
    phone_number: str
    type: NotRequired[Nullable[str]]
    r"""Kombo exposes type information through this field. If we don't get any information from the tool, we will set this to `null`."""


class GetAtsApplicationsPositiveResponsePhoneNumber(BaseModel):
    phone_number: str

    type: OptionalNullable[str] = UNSET
    r"""Kombo exposes type information through this field. If we don't get any information from the tool, we will set this to `null`."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["type"]
        nullable_fields = ["type"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class GetAtsApplicationsPositiveResponseSocialMediaTypedDict(TypedDict):
    link: NotRequired[Nullable[str]]
    type: NotRequired[Nullable[str]]
    username: NotRequired[Nullable[str]]


class GetAtsApplicationsPositiveResponseSocialMedia(BaseModel):
    link: OptionalNullable[str] = UNSET

    type: OptionalNullable[str] = UNSET

    username: OptionalNullable[str] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["link", "type", "username"]
        nullable_fields = ["link", "type", "username"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class GetAtsApplicationsPositiveResponseTagTypedDict(TypedDict):
    id: str
    r"""The globally unique ID of this object generated by Kombo. We recommend using this as a stable primary key for syncing."""
    remote_id: Nullable[str]
    r"""The raw ID of the object in the remote system. We don't recommend using this as a primary key on your side as it might sometimes be compromised of multiple identifiers if a system doesn't provide a clear primary key."""
    name: Nullable[str]


class GetAtsApplicationsPositiveResponseTag(BaseModel):
    id: str
    r"""The globally unique ID of this object generated by Kombo. We recommend using this as a stable primary key for syncing."""

    remote_id: Nullable[str]
    r"""The raw ID of the object in the remote system. We don't recommend using this as a primary key on your side as it might sometimes be compromised of multiple identifiers if a system doesn't provide a clear primary key."""

    name: Nullable[str]

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["remote_id", "name"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class GetAtsApplicationsPositiveResponseCandidateTypedDict(TypedDict):
    id: str
    r"""The globally unique ID of this object generated by Kombo. We recommend using this as a stable primary key for syncing."""
    remote_id: str
    r"""The raw ID of the object in the remote system. We don't recommend using this as a primary key on your side as it might sometimes be compromised of multiple identifiers if a system doesn't provide a clear primary key."""
    first_name: Nullable[str]
    r"""First name of the candidate."""
    last_name: Nullable[str]
    r"""Last name of the candidate."""
    source: Nullable[str]
    r"""The hiring source of the candidate. If you're a job board or recruiting service, you can use this to validate which candidates applied through your service and ensure that the correct referral compensation is paid out."""
    remote_url: Nullable[str]
    r"""URL to the candidate profile in the source ATS system. When embedding this in your system, please consider that not for all ATS a dedicated candidate profile URL is available, so you might need to fall back to the application URL."""
    tags: List[GetAtsApplicationsPositiveResponseTagTypedDict]
    email_addresses: NotRequired[
        Nullable[List[GetAtsApplicationsPositiveResponseEmailAddressTypedDict]]
    ]
    r"""A list of email addresses of the candidate with an optional type. If an email address is invalid, it will be filtered out."""
    phone_numbers: NotRequired[
        Nullable[List[GetAtsApplicationsPositiveResponsePhoneNumberTypedDict]]
    ]
    r"""A list of phone numbers of the candidate."""
    social_media: NotRequired[
        Nullable[List[GetAtsApplicationsPositiveResponseSocialMediaTypedDict]]
    ]
    r"""List of social media accounts of the candidate."""


class GetAtsApplicationsPositiveResponseCandidate(BaseModel):
    id: str
    r"""The globally unique ID of this object generated by Kombo. We recommend using this as a stable primary key for syncing."""

    remote_id: str
    r"""The raw ID of the object in the remote system. We don't recommend using this as a primary key on your side as it might sometimes be compromised of multiple identifiers if a system doesn't provide a clear primary key."""

    first_name: Nullable[str]
    r"""First name of the candidate."""

    last_name: Nullable[str]
    r"""Last name of the candidate."""

    source: Nullable[str]
    r"""The hiring source of the candidate. If you're a job board or recruiting service, you can use this to validate which candidates applied through your service and ensure that the correct referral compensation is paid out."""

    remote_url: Nullable[str]
    r"""URL to the candidate profile in the source ATS system. When embedding this in your system, please consider that not for all ATS a dedicated candidate profile URL is available, so you might need to fall back to the application URL."""

    tags: List[GetAtsApplicationsPositiveResponseTag]

    email_addresses: OptionalNullable[
        List[GetAtsApplicationsPositiveResponseEmailAddress]
    ] = UNSET
    r"""A list of email addresses of the candidate with an optional type. If an email address is invalid, it will be filtered out."""

    phone_numbers: OptionalNullable[
        List[GetAtsApplicationsPositiveResponsePhoneNumber]
    ] = UNSET
    r"""A list of phone numbers of the candidate."""

    social_media: OptionalNullable[
        List[GetAtsApplicationsPositiveResponseSocialMedia]
    ] = UNSET
    r"""List of social media accounts of the candidate."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["email_addresses", "phone_numbers", "social_media"]
        nullable_fields = [
            "first_name",
            "last_name",
            "email_addresses",
            "phone_numbers",
            "social_media",
            "source",
            "remote_url",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class GetAtsApplicationsPositiveResponseCurrentStageTypedDict(TypedDict):
    id: str
    r"""The globally unique ID of this object generated by Kombo. We recommend using this as a stable primary key for syncing."""
    remote_id: Nullable[str]
    r"""The raw ID of the object in the remote system. We don't recommend using this as a primary key on your side as it might sometimes be compromised of multiple identifiers if a system doesn't provide a clear primary key."""
    name: Nullable[str]
    r"""The application stage name. For example, \"Initial Screening\"."""
    index: Nullable[int]


class GetAtsApplicationsPositiveResponseCurrentStage(BaseModel):
    id: str
    r"""The globally unique ID of this object generated by Kombo. We recommend using this as a stable primary key for syncing."""

    remote_id: Nullable[str]
    r"""The raw ID of the object in the remote system. We don't recommend using this as a primary key on your side as it might sometimes be compromised of multiple identifiers if a system doesn't provide a clear primary key."""

    name: Nullable[str]
    r"""The application stage name. For example, \"Initial Screening\"."""

    index: Nullable[int]

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["remote_id", "name", "index"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class GetAtsApplicationsPositiveResponseJobTypedDict(TypedDict):
    id: str
    r"""The globally unique ID of this object generated by Kombo. We recommend using this as a stable primary key for syncing."""
    remote_id: str
    r"""The raw ID of the object in the remote system. We don't recommend using this as a primary key on your side as it might sometimes be compromised of multiple identifiers if a system doesn't provide a clear primary key."""
    name: Nullable[str]
    r"""Title of the job."""


class GetAtsApplicationsPositiveResponseJob(BaseModel):
    id: str
    r"""The globally unique ID of this object generated by Kombo. We recommend using this as a stable primary key for syncing."""

    remote_id: str
    r"""The raw ID of the object in the remote system. We don't recommend using this as a primary key on your side as it might sometimes be compromised of multiple identifiers if a system doesn't provide a clear primary key."""

    name: Nullable[str]
    r"""Title of the job."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["name"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class GetAtsApplicationsPositiveResponseLocationTypedDict(TypedDict):
    r"""Location of the interview."""

    city: NotRequired[Nullable[str]]
    country: NotRequired[Nullable[str]]
    r"""Contains the ISO2 country code if possible. If not, it contains the original value."""
    raw: NotRequired[Nullable[str]]
    r"""If we have address data, this is filled with the raw address string."""
    state: NotRequired[Nullable[str]]
    street_1: NotRequired[Nullable[str]]
    r"""If we can parse the address data, this field contains the first part of the street information."""
    street_2: NotRequired[Nullable[str]]
    zip_code: NotRequired[Nullable[str]]


class GetAtsApplicationsPositiveResponseLocation(BaseModel):
    r"""Location of the interview."""

    city: OptionalNullable[str] = UNSET

    country: OptionalNullable[str] = UNSET
    r"""Contains the ISO2 country code if possible. If not, it contains the original value."""

    raw: OptionalNullable[str] = UNSET
    r"""If we have address data, this is filled with the raw address string."""

    state: OptionalNullable[str] = UNSET

    street_1: OptionalNullable[str] = UNSET
    r"""If we can parse the address data, this field contains the first part of the street information."""

    street_2: OptionalNullable[str] = UNSET

    zip_code: OptionalNullable[str] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "city",
            "country",
            "raw",
            "state",
            "street_1",
            "street_2",
            "zip_code",
        ]
        nullable_fields = [
            "city",
            "country",
            "raw",
            "state",
            "street_1",
            "street_2",
            "zip_code",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class InterviewTypedDict(TypedDict):
    id: str
    r"""The globally unique ID of this object generated by Kombo. We recommend using this as a stable primary key for syncing."""
    remote_id: Nullable[str]
    r"""The raw ID of the object in the remote system. We don't recommend using this as a primary key on your side as it might sometimes be compromised of multiple identifiers if a system doesn't provide a clear primary key."""
    title: Nullable[str]
    r"""The title of the interview."""
    starting_at: Nullable[datetime]
    r"""The start time of the interview.
    https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/toISOString
    """
    ending_at: Nullable[datetime]
    r"""The end time of the interview.
    https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/toISOString
    """
    canceled: Nullable[bool]
    r"""Whether the interview was canceled or not. If we have no information, this field will be `null`."""
    location: NotRequired[Nullable[GetAtsApplicationsPositiveResponseLocationTypedDict]]
    r"""Location of the interview."""


class Interview(BaseModel):
    id: str
    r"""The globally unique ID of this object generated by Kombo. We recommend using this as a stable primary key for syncing."""

    remote_id: Nullable[str]
    r"""The raw ID of the object in the remote system. We don't recommend using this as a primary key on your side as it might sometimes be compromised of multiple identifiers if a system doesn't provide a clear primary key."""

    title: Nullable[str]
    r"""The title of the interview."""

    starting_at: Nullable[datetime]
    r"""The start time of the interview.
    https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/toISOString
    """

    ending_at: Nullable[datetime]
    r"""The end time of the interview.
    https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/toISOString
    """

    canceled: Nullable[bool]
    r"""Whether the interview was canceled or not. If we have no information, this field will be `null`."""

    location: OptionalNullable[GetAtsApplicationsPositiveResponseLocation] = UNSET
    r"""Location of the interview."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["location"]
        nullable_fields = [
            "remote_id",
            "title",
            "starting_at",
            "ending_at",
            "location",
            "canceled",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


GetAtsApplicationsPositiveResponseStatus = Literal[
    "ACCEPTED",
    "DECLINED",
    "SENT",
    "APPROVED",
    "DRAFT",
    "ABANDONED",
]
r"""The offer’s current status. The usual flow of statuses is as follows:
`DRAFT` -> `APPROVED` -> `SENT` -> `ACCEPTED` or `DECLINED`.

Please note that not all systems will expose all statuses. For example, most systems do not include the `APPROVED` status

- `ACCEPTED`: The offer was accepted by the candidate.
- `DECLINED`: The offer was declined by the candidate.
- `SENT`: The offer was sent to the candidate.
- `APPROVED`: The draft was approved.
- `DRAFT`: The offer is a draft and has not yet been sent to the candidate.
- `ABANDONED`: The offer has expired or is no longer valid and should not be considered.

"""


class OfferTypedDict(TypedDict):
    id: str
    r"""The globally unique ID of this object generated by Kombo. We recommend using this as a stable primary key for syncing."""
    remote_id: Nullable[str]
    r"""The raw ID of the object in the remote system. We don't recommend using this as a primary key on your side as it might sometimes be compromised of multiple identifiers if a system doesn't provide a clear primary key."""
    status: Nullable[GetAtsApplicationsPositiveResponseStatus]
    r"""The offer’s current status. The usual flow of statuses is as follows:
    `DRAFT` -> `APPROVED` -> `SENT` -> `ACCEPTED` or `DECLINED`.

    Please note that not all systems will expose all statuses. For example, most systems do not include the `APPROVED` status

    - `ACCEPTED`: The offer was accepted by the candidate.
    - `DECLINED`: The offer was declined by the candidate.
    - `SENT`: The offer was sent to the candidate.
    - `APPROVED`: The draft was approved.
    - `DRAFT`: The offer is a draft and has not yet been sent to the candidate.
    - `ABANDONED`: The offer has expired or is no longer valid and should not be considered.

    """


class Offer(BaseModel):
    id: str
    r"""The globally unique ID of this object generated by Kombo. We recommend using this as a stable primary key for syncing."""

    remote_id: Nullable[str]
    r"""The raw ID of the object in the remote system. We don't recommend using this as a primary key on your side as it might sometimes be compromised of multiple identifiers if a system doesn't provide a clear primary key."""

    status: Nullable[GetAtsApplicationsPositiveResponseStatus]
    r"""The offer’s current status. The usual flow of statuses is as follows:
    `DRAFT` -> `APPROVED` -> `SENT` -> `ACCEPTED` or `DECLINED`.

    Please note that not all systems will expose all statuses. For example, most systems do not include the `APPROVED` status

    - `ACCEPTED`: The offer was accepted by the candidate.
    - `DECLINED`: The offer was declined by the candidate.
    - `SENT`: The offer was sent to the candidate.
    - `APPROVED`: The draft was approved.
    - `DRAFT`: The offer is a draft and has not yet been sent to the candidate.
    - `ABANDONED`: The offer has expired or is no longer valid and should not be considered.

    """

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["remote_id", "status"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class GetAtsApplicationsPositiveResponseResultTypedDict(TypedDict):
    id: str
    r"""The globally unique ID of this object generated by Kombo. We recommend using this as a stable primary key for syncing."""
    remote_id: Nullable[str]
    r"""The raw ID of the object in the remote system. We don't recommend using this as a primary key on your side as it might sometimes be compromised of multiple identifiers if a system doesn't provide a clear primary key."""
    outcome: Nullable[GetAtsApplicationsPositiveResponseOutcome]
    r"""Parsed status of the application. If Kombo identifies that the application was accepted and the candidate hired, it will be `HIRED`. If the application was rejected or the candidate declined, it will be `DECLINED`. If the application is still in process, it will be `PENDING`.
    Kombo will always try to deliver this information as reliably as possible.
    """
    rejection_reason_name: Nullable[str]
    r"""Reason for the rejection of the candidate."""
    rejected_at: Nullable[datetime]
    r"""The time that the application was rejected.
    https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/toISOString
    """
    current_stage_id: Nullable[str]
    r"""ID of the current application stage"""
    job_id: Nullable[str]
    r"""The Kombo ID of the job which the candidate applied to. The ID can be used to retrieve the job from the `get jobs` endpoint."""
    candidate_id: Nullable[str]
    r"""The Kombo ID of the candidate who applied to the job. The ID can be used to retrieve the candidate from the `get candidates` endpoint."""
    custom_fields: Nullable[Dict[str, Any]]
    r"""A key-value store of fields not covered by the schema. [Read more](/custom-fields)"""
    remote_url: Nullable[str]
    r"""URL to the application in the source ATS system. When embedding this in your system, please consider that not for all ATS a dedicated application URL is available, so you might need to fall back to the candidate URL."""
    changed_at: datetime
    r"""The timestamp when this specific record was last modified. This field only updates when properties directly on this record change, NOT when related or nested models change. For filtering that considers nested data changes, use the `updated_after` parameter which will return records when either the record itself OR its related models have been updated.
    https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/toISOString
    """
    remote_deleted_at: Nullable[datetime]
    r"""The date and time the object was deleted in the remote system. Objects are automatically marked as deleted when Kombo can't retrieve them from the remote system anymore. Kombo will also anonymize entries 14 days after they disappear.
    https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/toISOString
    """
    remote_created_at: Nullable[datetime]
    r"""The date and time the object was created in the remote system.
    https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/toISOString
    """
    remote_updated_at: Nullable[datetime]
    r"""A timestamp retrieved from the remote system, describing when the resource was last updated.
    https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/toISOString
    """
    candidate: Nullable[GetAtsApplicationsPositiveResponseCandidateTypedDict]
    current_stage: Nullable[GetAtsApplicationsPositiveResponseCurrentStageTypedDict]
    job: Nullable[GetAtsApplicationsPositiveResponseJobTypedDict]
    interviews: List[InterviewTypedDict]
    offers: List[OfferTypedDict]
    screening_question_answers: NotRequired[
        Nullable[
            List[
                GetAtsApplicationsPositiveResponseScreeningQuestionAnswerUnionTypedDict
            ]
        ]
    ]
    r"""A list of answers to screening questions. The screening answer type `FILE` is currently unsupported."""


class GetAtsApplicationsPositiveResponseResult(BaseModel):
    id: str
    r"""The globally unique ID of this object generated by Kombo. We recommend using this as a stable primary key for syncing."""

    remote_id: Nullable[str]
    r"""The raw ID of the object in the remote system. We don't recommend using this as a primary key on your side as it might sometimes be compromised of multiple identifiers if a system doesn't provide a clear primary key."""

    outcome: Nullable[GetAtsApplicationsPositiveResponseOutcome]
    r"""Parsed status of the application. If Kombo identifies that the application was accepted and the candidate hired, it will be `HIRED`. If the application was rejected or the candidate declined, it will be `DECLINED`. If the application is still in process, it will be `PENDING`.
    Kombo will always try to deliver this information as reliably as possible.
    """

    rejection_reason_name: Nullable[str]
    r"""Reason for the rejection of the candidate."""

    rejected_at: Nullable[datetime]
    r"""The time that the application was rejected.
    https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/toISOString
    """

    current_stage_id: Nullable[str]
    r"""ID of the current application stage"""

    job_id: Nullable[str]
    r"""The Kombo ID of the job which the candidate applied to. The ID can be used to retrieve the job from the `get jobs` endpoint."""

    candidate_id: Nullable[str]
    r"""The Kombo ID of the candidate who applied to the job. The ID can be used to retrieve the candidate from the `get candidates` endpoint."""

    custom_fields: Nullable[Dict[str, Any]]
    r"""A key-value store of fields not covered by the schema. [Read more](/custom-fields)"""

    remote_url: Nullable[str]
    r"""URL to the application in the source ATS system. When embedding this in your system, please consider that not for all ATS a dedicated application URL is available, so you might need to fall back to the candidate URL."""

    changed_at: datetime
    r"""The timestamp when this specific record was last modified. This field only updates when properties directly on this record change, NOT when related or nested models change. For filtering that considers nested data changes, use the `updated_after` parameter which will return records when either the record itself OR its related models have been updated.
    https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/toISOString
    """

    remote_deleted_at: Nullable[datetime]
    r"""The date and time the object was deleted in the remote system. Objects are automatically marked as deleted when Kombo can't retrieve them from the remote system anymore. Kombo will also anonymize entries 14 days after they disappear.
    https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/toISOString
    """

    remote_created_at: Nullable[datetime]
    r"""The date and time the object was created in the remote system.
    https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/toISOString
    """

    remote_updated_at: Nullable[datetime]
    r"""A timestamp retrieved from the remote system, describing when the resource was last updated.
    https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/toISOString
    """

    candidate: Nullable[GetAtsApplicationsPositiveResponseCandidate]

    current_stage: Nullable[GetAtsApplicationsPositiveResponseCurrentStage]

    job: Nullable[GetAtsApplicationsPositiveResponseJob]

    interviews: List[Interview]

    offers: List[Offer]

    screening_question_answers: OptionalNullable[
        List[GetAtsApplicationsPositiveResponseScreeningQuestionAnswerUnion]
    ] = UNSET
    r"""A list of answers to screening questions. The screening answer type `FILE` is currently unsupported."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["screening_question_answers"]
        nullable_fields = [
            "remote_id",
            "outcome",
            "rejection_reason_name",
            "rejected_at",
            "current_stage_id",
            "job_id",
            "candidate_id",
            "screening_question_answers",
            "custom_fields",
            "remote_url",
            "remote_deleted_at",
            "remote_created_at",
            "remote_updated_at",
            "candidate",
            "current_stage",
            "job",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class GetAtsApplicationsPositiveResponseDataTypedDict(TypedDict):
    next: Nullable[str]
    r"""Cursor string that can be passed to the `cursor` query parameter to get the next page. If this is `null`, then there are no more pages."""
    results: List[GetAtsApplicationsPositiveResponseResultTypedDict]


class GetAtsApplicationsPositiveResponseData(BaseModel):
    next: Nullable[str]
    r"""Cursor string that can be passed to the `cursor` query parameter to get the next page. If this is `null`, then there are no more pages."""

    results: List[GetAtsApplicationsPositiveResponseResult]

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["next"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class GetAtsApplicationsPositiveResponseTypedDict(TypedDict):
    data: GetAtsApplicationsPositiveResponseDataTypedDict
    status: Literal["success"]


class GetAtsApplicationsPositiveResponse(BaseModel):
    data: GetAtsApplicationsPositiveResponseData

    STATUS: Annotated[
        Annotated[Literal["success"], AfterValidator(validate_const("success"))],
        pydantic.Field(alias="status"),
    ] = "success"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from typing import Dict, List, Union
from typing_extensions import TypeAliasType


Schema4TypedDict = TypeAliasType(
    "Schema4TypedDict",
    Union[str, float, bool, Dict[str, "Schema4TypedDict"], List["Schema4TypedDict"]],
)


Schema4 = TypeAliasType(
    "Schema4", Union[str, float, bool, Dict[str, "Schema4"], List["Schema4"]]
)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from kombo.types import BaseModel
from typing import List, Literal
from typing_extensions import TypedDict


DataChangedWebhookPayloadType = Literal["data-changed",]
r"""Type of the webhook event"""


DataChangedWebhookPayloadIntegrationCategory = Literal[
    "HRIS",
    "ATS",
    "ASSESSMENT",
    "LMS",
]
r"""The category of the integration (HRIS, ATS, ASSESSMENT, or LMS)."""


Name = Literal[
    "hris_legal_entities",
    "hris_locations",
    "hris_employees",
    "hris_absence_types",
    "hris_absences",
    "hris_employments",
    "hris_teams",
    "hris_time_off_balances",
    "hris_timesheets",
    "hris_employee_document_categories",
    "hris_performance_reviews",
    "hris_performance_review_cycles",
    "ats_users",
    "ats_jobs",
    "ats_job_postings",
    "ats_candidates",
    "ats_application_stages",
    "ats_applications",
    "ats_screening_questions",
    "ats_tags",
    "ats_interviews",
    "ats_offers",
    "ats_rejection_reasons",
    "lms_users",
    "lms_course_providers",
    "lms_courses",
    "lms_course_progressions",
    "lms_course_revisions",
    "lms_skills",
    "hris_join_employees_teams",
    "ats_join_candidates_tags",
    "ats_join_jobs_application_stages",
    "ats_join_jobs_screening_questions",
    "ats_join_jobs_users",
    "ats_join_interviews_users",
    "lms_join_revisions_skills",
]
r"""The name of the Kombo model or connection that changed."""


class ChangedModelTypedDict(TypedDict):
    r"""A model or connection that has changed in the integration."""

    name: Name
    r"""The name of the Kombo model or connection that changed."""


class ChangedModel(BaseModel):
    r"""A model or connection that has changed in the integration."""

    name: Name
    r"""The name of the Kombo model or connection that changed."""


class DataChangedWebhookPayloadDataTypedDict(TypedDict):
    integration_id: str
    r"""The unique identifier of the integration."""
    integration_tool: str
    r"""The name of the integrated tool (e.g., \"personio\", \"greenhouse\")."""
    integration_category: DataChangedWebhookPayloadIntegrationCategory
    r"""The category of the integration (HRIS, ATS, ASSESSMENT, or LMS)."""
    changed_models: List[ChangedModelTypedDict]
    r"""List of data models and connections for which one or more records were created, updated, or deleted."""


class DataChangedWebhookPayloadData(BaseModel):
    integration_id: str
    r"""The unique identifier of the integration."""

    integration_tool: str
    r"""The name of the integrated tool (e.g., \"personio\", \"greenhouse\")."""

    integration_category: DataChangedWebhookPayloadIntegrationCategory
    r"""The category of the integration (HRIS, ATS, ASSESSMENT, or LMS)."""

    changed_models: List[ChangedModel]
    r"""List of data models and connections for which one or more records were created, updated, or deleted."""


class DataChangedWebhookPayloadTypedDict(TypedDict):
    id: str
    r"""Unique identifier for this webhook event"""
    type: DataChangedWebhookPayloadType
    r"""Type of the webhook event"""
    data: DataChangedWebhookPayloadDataTypedDict


class DataChangedWebhookPayload(BaseModel):
    id: str
    r"""Unique identifier for this webhook event"""

    type: DataChangedWebhookPayloadType
    r"""Type of the webhook event"""

    data: DataChangedWebhookPayloadData



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .schema1 import Schema1, Schema1TypedDict
from kombo.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from kombo.utils import validate_const
import pydantic
from pydantic import Field, model_serializer
from pydantic.functional_validators import AfterValidator
from typing import List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


Schema2UnifiedKey8 = Literal[
    "first_name",
    "last_name",
    "date_of_birth",
    "gender",
    "home_address.city",
    "home_address.country",
    "home_address.state",
    "home_address.street_1",
    "home_address.street_2",
    "home_address.zip_code",
    "job_title",
    "legal_entity_id",
    "location_id",
    "mobile_phone_number",
    "home_phone_number",
    "nationality",
    "start_date",
    "work_email",
    "private_email",
    "yearly_salary",
]


class Schema2FileRestrictionsTypedDict(TypedDict):
    accepted_mime_types: List[str]
    max_file_size: NotRequired[Nullable[float]]


class Schema2FileRestrictions(BaseModel):
    accepted_mime_types: List[str]

    max_file_size: OptionalNullable[float] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["max_file_size"]
        nullable_fields = ["max_file_size"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class Schema2FileTypedDict(TypedDict):
    label: str
    required: bool
    file_restrictions: Schema2FileRestrictionsTypedDict
    description: NotRequired[Nullable[str]]
    unified_key: NotRequired[Nullable[Schema2UnifiedKey8]]
    type: Literal["file"]


class Schema2File(BaseModel):
    label: str

    required: bool

    file_restrictions: Schema2FileRestrictions

    description: OptionalNullable[str] = UNSET

    unified_key: OptionalNullable[Schema2UnifiedKey8] = UNSET

    TYPE: Annotated[
        Annotated[Literal["file"], AfterValidator(validate_const("file"))],
        pydantic.Field(alias="type"),
    ] = "file"

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["description", "unified_key"]
        nullable_fields = ["description", "unified_key"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


Schema2UnifiedKey7 = Literal[
    "first_name",
    "last_name",
    "date_of_birth",
    "gender",
    "home_address.city",
    "home_address.country",
    "home_address.state",
    "home_address.street_1",
    "home_address.street_2",
    "home_address.zip_code",
    "job_title",
    "legal_entity_id",
    "location_id",
    "mobile_phone_number",
    "home_phone_number",
    "nationality",
    "start_date",
    "work_email",
    "private_email",
    "yearly_salary",
]


Schema2UnifiedKey1 = Literal[
    "first_name",
    "last_name",
    "date_of_birth",
    "gender",
    "home_address.city",
    "home_address.country",
    "home_address.state",
    "home_address.street_1",
    "home_address.street_2",
    "home_address.zip_code",
    "job_title",
    "legal_entity_id",
    "location_id",
    "mobile_phone_number",
    "home_phone_number",
    "nationality",
    "start_date",
    "work_email",
    "private_email",
    "yearly_salary",
]


class Schema2TextTypedDict(TypedDict):
    label: str
    required: bool
    description: NotRequired[Nullable[str]]
    unified_key: NotRequired[Nullable[Schema2UnifiedKey1]]
    type: Literal["text"]
    min_length: NotRequired[Nullable[float]]
    max_length: NotRequired[Nullable[float]]
    reg_exp: NotRequired[Nullable[str]]


class Schema2Text(BaseModel):
    label: str

    required: bool

    description: OptionalNullable[str] = UNSET

    unified_key: OptionalNullable[Schema2UnifiedKey1] = UNSET

    TYPE: Annotated[
        Annotated[Literal["text"], AfterValidator(validate_const("text"))],
        pydantic.Field(alias="type"),
    ] = "text"

    min_length: OptionalNullable[float] = UNSET

    max_length: OptionalNullable[float] = UNSET

    reg_exp: OptionalNullable[str] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "description",
            "unified_key",
            "min_length",
            "max_length",
            "reg_exp",
        ]
        nullable_fields = [
            "description",
            "unified_key",
            "min_length",
            "max_length",
            "reg_exp",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


Schema2UnifiedKey2 = Literal[
    "first_name",
    "last_name",
    "date_of_birth",
    "gender",
    "home_address.city",
    "home_address.country",
    "home_address.state",
    "home_address.street_1",
    "home_address.street_2",
    "home_address.zip_code",
    "job_title",
    "legal_entity_id",
    "location_id",
    "mobile_phone_number",
    "home_phone_number",
    "nationality",
    "start_date",
    "work_email",
    "private_email",
    "yearly_salary",
]


class Schema2NumberTypedDict(TypedDict):
    label: str
    required: bool
    description: NotRequired[Nullable[str]]
    unified_key: NotRequired[Nullable[Schema2UnifiedKey2]]
    type: Literal["number"]
    min: NotRequired[Nullable[float]]
    max: NotRequired[Nullable[float]]


class Schema2Number(BaseModel):
    label: str

    required: bool

    description: OptionalNullable[str] = UNSET

    unified_key: OptionalNullable[Schema2UnifiedKey2] = UNSET

    TYPE: Annotated[
        Annotated[Literal["number"], AfterValidator(validate_const("number"))],
        pydantic.Field(alias="type"),
    ] = "number"

    min: OptionalNullable[float] = UNSET

    max: OptionalNullable[float] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["description", "unified_key", "min", "max"]
        nullable_fields = ["description", "unified_key", "min", "max"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


Schema2UnifiedKey3 = Literal[
    "first_name",
    "last_name",
    "date_of_birth",
    "gender",
    "home_address.city",
    "home_address.country",
    "home_address.state",
    "home_address.street_1",
    "home_address.street_2",
    "home_address.zip_code",
    "job_title",
    "legal_entity_id",
    "location_id",
    "mobile_phone_number",
    "home_phone_number",
    "nationality",
    "start_date",
    "work_email",
    "private_email",
    "yearly_salary",
]


class Schema2DateTypedDict(TypedDict):
    label: str
    required: bool
    description: NotRequired[Nullable[str]]
    unified_key: NotRequired[Nullable[Schema2UnifiedKey3]]
    type: Literal["date"]


class Schema2Date(BaseModel):
    label: str

    required: bool

    description: OptionalNullable[str] = UNSET

    unified_key: OptionalNullable[Schema2UnifiedKey3] = UNSET

    TYPE: Annotated[
        Annotated[Literal["date"], AfterValidator(validate_const("date"))],
        pydantic.Field(alias="type"),
    ] = "date"

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["description", "unified_key"]
        nullable_fields = ["description", "unified_key"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


Schema2UnifiedKey4 = Literal[
    "first_name",
    "last_name",
    "date_of_birth",
    "gender",
    "home_address.city",
    "home_address.country",
    "home_address.state",
    "home_address.street_1",
    "home_address.street_2",
    "home_address.zip_code",
    "job_title",
    "legal_entity_id",
    "location_id",
    "mobile_phone_number",
    "home_phone_number",
    "nationality",
    "start_date",
    "work_email",
    "private_email",
    "yearly_salary",
]


Schema2RemoteID1TypedDict = TypeAliasType(
    "Schema2RemoteID1TypedDict", Union[str, float]
)


Schema2RemoteID1 = TypeAliasType("Schema2RemoteID1", Union[str, float])


class Schema2Entry1TypedDict(TypedDict):
    id: str
    label: str
    remote_id: Schema2RemoteID1TypedDict
    unified_value: NotRequired[str]


class Schema2Entry1(BaseModel):
    id: str

    label: str

    remote_id: Schema2RemoteID1

    unified_value: Optional[str] = None


class Schema2OptionsInline1TypedDict(TypedDict):
    entries: List[Schema2Entry1TypedDict]
    type: Literal["inline"]


class Schema2OptionsInline1(BaseModel):
    entries: List[Schema2Entry1]

    TYPE: Annotated[
        Annotated[Literal["inline"], AfterValidator(validate_const("inline"))],
        pydantic.Field(alias="type"),
    ] = "inline"


class Schema2OptionsReferenced1TypedDict(TypedDict):
    link: str
    type: Literal["referenced"]


class Schema2OptionsReferenced1(BaseModel):
    link: str

    TYPE: Annotated[
        Annotated[Literal["referenced"], AfterValidator(validate_const("referenced"))],
        pydantic.Field(alias="type"),
    ] = "referenced"


Schema2OptionsUnion1TypedDict = TypeAliasType(
    "Schema2OptionsUnion1TypedDict",
    Union[Schema2OptionsInline1TypedDict, Schema2OptionsReferenced1TypedDict],
)


Schema2OptionsUnion1 = Annotated[
    Union[Schema2OptionsInline1, Schema2OptionsReferenced1], Field(discriminator="TYPE")
]


class Schema2SingleSelectTypedDict(TypedDict):
    label: str
    required: bool
    options: Schema2OptionsUnion1TypedDict
    description: NotRequired[Nullable[str]]
    unified_key: NotRequired[Nullable[Schema2UnifiedKey4]]
    type: Literal["single_select"]


class Schema2SingleSelect(BaseModel):
    label: str

    required: bool

    options: Schema2OptionsUnion1

    description: OptionalNullable[str] = UNSET

    unified_key: OptionalNullable[Schema2UnifiedKey4] = UNSET

    TYPE: Annotated[
        Annotated[
            Literal["single_select"], AfterValidator(validate_const("single_select"))
        ],
        pydantic.Field(alias="type"),
    ] = "single_select"

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["description", "unified_key"]
        nullable_fields = ["description", "unified_key"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


Schema2RemoteID2TypedDict = TypeAliasType(
    "Schema2RemoteID2TypedDict", Union[str, float]
)


Schema2RemoteID2 = TypeAliasType("Schema2RemoteID2", Union[str, float])


class Schema2Entry2TypedDict(TypedDict):
    id: str
    label: str
    remote_id: Schema2RemoteID2TypedDict
    unified_value: NotRequired[str]


class Schema2Entry2(BaseModel):
    id: str

    label: str

    remote_id: Schema2RemoteID2

    unified_value: Optional[str] = None


class Schema2OptionsInline2TypedDict(TypedDict):
    entries: List[Schema2Entry2TypedDict]
    type: Literal["inline"]


class Schema2OptionsInline2(BaseModel):
    entries: List[Schema2Entry2]

    TYPE: Annotated[
        Annotated[Literal["inline"], AfterValidator(validate_const("inline"))],
        pydantic.Field(alias="type"),
    ] = "inline"


class Schema2OptionsReferenced2TypedDict(TypedDict):
    link: str
    type: Literal["referenced"]


class Schema2OptionsReferenced2(BaseModel):
    link: str

    TYPE: Annotated[
        Annotated[Literal["referenced"], AfterValidator(validate_const("referenced"))],
        pydantic.Field(alias="type"),
    ] = "referenced"


Schema2OptionsUnion2TypedDict = TypeAliasType(
    "Schema2OptionsUnion2TypedDict",
    Union[Schema2OptionsInline2TypedDict, Schema2OptionsReferenced2TypedDict],
)


Schema2OptionsUnion2 = Annotated[
    Union[Schema2OptionsInline2, Schema2OptionsReferenced2], Field(discriminator="TYPE")
]


class Schema2MultiSelectTypedDict(TypedDict):
    label: str
    required: bool
    options: Schema2OptionsUnion2TypedDict
    description: NotRequired[Nullable[str]]
    unified_key: NotRequired[Nullable[str]]
    type: Literal["multi_select"]
    min_items: NotRequired[Nullable[float]]
    max_items: NotRequired[Nullable[float]]


class Schema2MultiSelect(BaseModel):
    label: str

    required: bool

    options: Schema2OptionsUnion2

    description: OptionalNullable[str] = UNSET

    unified_key: OptionalNullable[str] = UNSET

    TYPE: Annotated[
        Annotated[
            Literal["multi_select"], AfterValidator(validate_const("multi_select"))
        ],
        pydantic.Field(alias="type"),
    ] = "multi_select"

    min_items: OptionalNullable[float] = UNSET

    max_items: OptionalNullable[float] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["description", "unified_key", "min_items", "max_items"]
        nullable_fields = ["description", "unified_key", "min_items", "max_items"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


Schema2UnifiedKey5 = Literal[
    "first_name",
    "last_name",
    "date_of_birth",
    "gender",
    "home_address.city",
    "home_address.country",
    "home_address.state",
    "home_address.street_1",
    "home_address.street_2",
    "home_address.zip_code",
    "job_title",
    "legal_entity_id",
    "location_id",
    "mobile_phone_number",
    "home_phone_number",
    "nationality",
    "start_date",
    "work_email",
    "private_email",
    "yearly_salary",
]


class Schema2CheckboxTypedDict(TypedDict):
    label: str
    required: bool
    description: NotRequired[Nullable[str]]
    unified_key: NotRequired[Nullable[Schema2UnifiedKey5]]
    type: Literal["checkbox"]


class Schema2Checkbox(BaseModel):
    label: str

    required: bool

    description: OptionalNullable[str] = UNSET

    unified_key: OptionalNullable[Schema2UnifiedKey5] = UNSET

    TYPE: Annotated[
        Annotated[Literal["checkbox"], AfterValidator(validate_const("checkbox"))],
        pydantic.Field(alias="type"),
    ] = "checkbox"

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["description", "unified_key"]
        nullable_fields = ["description", "unified_key"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


Schema2UnifiedKey6 = Literal[
    "first_name",
    "last_name",
    "date_of_birth",
    "gender",
    "home_address.city",
    "home_address.country",
    "home_address.state",
    "home_address.street_1",
    "home_address.street_2",
    "home_address.zip_code",
    "job_title",
    "legal_entity_id",
    "location_id",
    "mobile_phone_number",
    "home_phone_number",
    "nationality",
    "start_date",
    "work_email",
    "private_email",
    "yearly_salary",
]


class Schema2Object1TypedDict(TypedDict):
    label: str
    required: bool
    properties: Schema1TypedDict
    description: NotRequired[Nullable[str]]
    unified_key: NotRequired[Nullable[Schema2UnifiedKey6]]
    type: Literal["object"]


class Schema2Object1(BaseModel):
    label: str

    required: bool

    properties: Schema1

    description: OptionalNullable[str] = UNSET

    unified_key: OptionalNullable[Schema2UnifiedKey6] = UNSET

    TYPE: Annotated[
        Annotated[Literal["object"], AfterValidator(validate_const("object"))],
        pydantic.Field(alias="type"),
    ] = "object"

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["description", "unified_key"]
        nullable_fields = ["description", "unified_key"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class Schema2Array1TypedDict(TypedDict):
    label: str
    required: bool
    item_type: Schema2Union1TypedDict
    description: NotRequired[Nullable[str]]
    unified_key: NotRequired[Nullable[Schema2UnifiedKey7]]
    type: Literal["array"]
    min_items: NotRequired[Nullable[float]]
    max_items: NotRequired[Nullable[float]]


class Schema2Array1(BaseModel):
    label: str

    required: bool

    item_type: Schema2Union1

    description: OptionalNullable[str] = UNSET

    unified_key: OptionalNullable[Schema2UnifiedKey7] = UNSET

    TYPE: Annotated[
        Annotated[Literal["array"], AfterValidator(validate_const("array"))],
        pydantic.Field(alias="type"),
    ] = "array"

    min_items: OptionalNullable[float] = UNSET

    max_items: OptionalNullable[float] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["description", "unified_key", "min_items", "max_items"]
        nullable_fields = ["description", "unified_key", "min_items", "max_items"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


Schema2Union1TypedDict = TypeAliasType(
    "Schema2Union1TypedDict",
    Union[
        Schema2DateTypedDict,
        Schema2CheckboxTypedDict,
        Schema2SingleSelectTypedDict,
        Schema2Object1TypedDict,
        Schema2FileTypedDict,
        Schema2NumberTypedDict,
        Schema2TextTypedDict,
        Schema2MultiSelectTypedDict,
        "Schema2Array1TypedDict",
    ],
)


Schema2Union1 = Annotated[
    Union[
        Schema2Text,
        Schema2Number,
        Schema2Date,
        Schema2SingleSelect,
        Schema2MultiSelect,
        Schema2Checkbox,
        Schema2Object1,
        "Schema2Array1",
        Schema2File,
    ],
    Field(discriminator="TYPE"),
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .schema1_union_1 import (
    Schema1Checkbox,
    Schema1CheckboxTypedDict,
    Schema1Date,
    Schema1DateTypedDict,
    Schema1File,
    Schema1FileTypedDict,
    Schema1MultiSelect,
    Schema1MultiSelectTypedDict,
    Schema1Number,
    Schema1NumberTypedDict,
    Schema1Object,
    Schema1ObjectTypedDict,
    Schema1SingleSelect,
    Schema1SingleSelectTypedDict,
    Schema1Text,
    Schema1TextTypedDict,
    Schema1UnifiedKey7,
)
from kombo.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from kombo.utils import validate_const
import pydantic
from pydantic import Field, model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Literal, TYPE_CHECKING, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict

if TYPE_CHECKING:
    from .schema2_union_2 import Schema2Union2, Schema2Union2TypedDict


Schema1Union2TypedDict = TypeAliasType(
    "Schema1Union2TypedDict",
    Union[
        Schema1DateTypedDict,
        Schema1CheckboxTypedDict,
        Schema1SingleSelectTypedDict,
        Schema1ObjectTypedDict,
        Schema1FileTypedDict,
        Schema1NumberTypedDict,
        Schema1TextTypedDict,
        Schema1MultiSelectTypedDict,
        "Schema1Array2TypedDict",
    ],
)


Schema1Union2 = Annotated[
    Union[
        Schema1Text,
        Schema1Number,
        Schema1Date,
        Schema1SingleSelect,
        Schema1MultiSelect,
        Schema1Checkbox,
        Schema1Object,
        "Schema1Array2",
        Schema1File,
    ],
    Field(discriminator="TYPE"),
]


class Schema1Array2TypedDict(TypedDict):
    label: str
    required: bool
    item_type: "Schema2Union2TypedDict"
    description: NotRequired[Nullable[str]]
    unified_key: NotRequired[Nullable[Schema1UnifiedKey7]]
    type: Literal["array"]
    min_items: NotRequired[Nullable[float]]
    max_items: NotRequired[Nullable[float]]


class Schema1Array2(BaseModel):
    label: str

    required: bool

    item_type: "Schema2Union2"

    description: OptionalNullable[str] = UNSET

    unified_key: OptionalNullable[Schema1UnifiedKey7] = UNSET

    TYPE: Annotated[
        Annotated[Literal["array"], AfterValidator(validate_const("array"))],
        pydantic.Field(alias="type"),
    ] = "array"

    min_items: OptionalNullable[float] = UNSET

    max_items: OptionalNullable[float] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["description", "unified_key", "min_items", "max_items"]
        nullable_fields = ["description", "unified_key", "min_items", "max_items"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from kombo.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class DocumentTypedDict(TypedDict):
    name: str
    r"""File name of the file you want to upload. We recommend providing something human-readable with a valid file extension (like `Resume.pdf`), as this might be shown in your customer's system."""
    content_type: NotRequired[str]
    r"""Content/MIME type of the file (e.g., `application/pdf`).

    If you provide `data`, this is required. If you provide `data_url`, this is optional and we'll attempt to use the `Content-Type` header of the response.

    **Note:** Please validate that the content type you provide is actually meaningful (and not something generic like [`application/octet-stream`](https://developer.mozilla.org/en-US/docs/Web/HTTP/Guides/MIME_types#applicationoctet-stream)). Especially when using object storage (like AWS S3), it's easy to accidentally discard the content types of user-provided files, so make sure to explicitly persist them when processing uploads from your users (see guides for [AWS](https://docs.aws.amazon.com/AmazonS3/latest/userguide/UsingMetadata.html#SysMetadata), [Google Cloud](https://cloud.google.com/storage/docs/metadata#content-type), and [Azure](https://learn.microsoft.com/en-us/rest/api/storageservices/put-blob#request-headers-all-blob-types)).
    """
    data_url: NotRequired[str]
    r"""URL to the file you want to upload that can be accessed without authentication headers. We will download the content immediately when receiving the request, so the URL can be short-lived.

    If you're using an object storage provider (like AWS S3), we strongly recommend providing a signed URL for secure access (see guides for [AWS](https://docs.aws.amazon.com/AmazonS3/latest/userguide/ShareObjectPreSignedURL.html), [Google Cloud](https://cloud.google.com/storage/docs/access-control/signed-urls), and [Azure](https://learn.microsoft.com/en-us/azure/storage/common/storage-sas-overview)).

    **Note:** You must provide either this or `data_url`. We recommend `data_url` over `data` for most cases.
    """
    data: NotRequired[str]
    r"""Base64-encoded contents of the file you want to upload.

    **Note:** You must provide either this or `data_url`. We recommend `data_url` over `data` for most cases.
    """


class Document(BaseModel):
    name: str
    r"""File name of the file you want to upload. We recommend providing something human-readable with a valid file extension (like `Resume.pdf`), as this might be shown in your customer's system."""

    content_type: Optional[str] = None
    r"""Content/MIME type of the file (e.g., `application/pdf`).

    If you provide `data`, this is required. If you provide `data_url`, this is optional and we'll attempt to use the `Content-Type` header of the response.

    **Note:** Please validate that the content type you provide is actually meaningful (and not something generic like [`application/octet-stream`](https://developer.mozilla.org/en-US/docs/Web/HTTP/Guides/MIME_types#applicationoctet-stream)). Especially when using object storage (like AWS S3), it's easy to accidentally discard the content types of user-provided files, so make sure to explicitly persist them when processing uploads from your users (see guides for [AWS](https://docs.aws.amazon.com/AmazonS3/latest/userguide/UsingMetadata.html#SysMetadata), [Google Cloud](https://cloud.google.com/storage/docs/metadata#content-type), and [Azure](https://learn.microsoft.com/en-us/rest/api/storageservices/put-blob#request-headers-all-blob-types)).
    """

    data_url: Optional[str] = None
    r"""URL to the file you want to upload that can be accessed without authentication headers. We will download the content immediately when receiving the request, so the URL can be short-lived.

    If you're using an object storage provider (like AWS S3), we strongly recommend providing a signed URL for secure access (see guides for [AWS](https://docs.aws.amazon.com/AmazonS3/latest/userguide/ShareObjectPreSignedURL.html), [Google Cloud](https://cloud.google.com/storage/docs/access-control/signed-urls), and [Azure](https://learn.microsoft.com/en-us/azure/storage/common/storage-sas-overview)).

    **Note:** You must provide either this or `data_url`. We recommend `data_url` over `data` for most cases.
    """

    data: Optional[str] = None
    r"""Base64-encoded contents of the file you want to upload.

    **Note:** You must provide either this or `data_url`. We recommend `data_url` over `data` for most cases.
    """


class PostHrisEmployeesEmployeeIDDocumentsRequestBodyTypedDict(TypedDict):
    category_id: str
    document: DocumentTypedDict


class PostHrisEmployeesEmployeeIDDocumentsRequestBody(BaseModel):
    category_id: str

    document: Document



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .patchintegrationsintegrationidintegrationfieldsintegrationfieldidrequestbody import (
    PatchIntegrationsIntegrationIDIntegrationFieldsIntegrationFieldIDRequestBody,
    PatchIntegrationsIntegrationIDIntegrationFieldsIntegrationFieldIDRequestBodyTypedDict,
)
from kombo.types import BaseModel
from kombo.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing_extensions import Annotated, TypedDict


class PatchIntegrationsIntegrationIDIntegrationFieldsIntegrationFieldIDRequestTypedDict(
    TypedDict
):
    integration_id: str
    r"""PATCH /integrations/:integration_id/integration-fields/:integration_field_id Parameter"""
    integration_field_id: str
    r"""PATCH /integrations/:integration_id/integration-fields/:integration_field_id Parameter"""
    body: PatchIntegrationsIntegrationIDIntegrationFieldsIntegrationFieldIDRequestBodyTypedDict
    r"""PATCH /integrations/:integration_id/integration-fields/:integration_field_id Request body"""


class PatchIntegrationsIntegrationIDIntegrationFieldsIntegrationFieldIDRequest(
    BaseModel
):
    integration_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""PATCH /integrations/:integration_id/integration-fields/:integration_field_id Parameter"""

    integration_field_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""PATCH /integrations/:integration_id/integration-fields/:integration_field_id Parameter"""

    body: Annotated[
        PatchIntegrationsIntegrationIDIntegrationFieldsIntegrationFieldIDRequestBody,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]
    r"""PATCH /integrations/:integration_id/integration-fields/:integration_field_id Request body"""



"""Tests for Kombo ATS Jobs API."""

from inline_snapshot import snapshot
from tests.conftest import MockContext


class TestKomboATSJobsAPI:
    """Test Kombo ATS Jobs API."""

    def test_should_make_correct_http_request_for_get_jobs(self):
        """Test that get_jobs makes correct HTTP request."""
        ctx = MockContext()

        # Mock the API endpoint
        ctx.mock_endpoint(
            method="GET",
            path="/v1/ats/jobs",
            response={
                "body": {
                    "status": "success",
                    "data": {
                        "results": [],
                        "next": None,
                    },
                },
            },
        )

        # Make the API call
        jobs = ctx.kombo.ats.get_jobs()
        if jobs is not None:
            _ = jobs.next()  # Consume first page

        # Verify and snapshot the request details
        request = ctx.get_last_request()
        assert request.path == snapshot('/v1/ats/jobs?page_size=100&include_deleted=false')




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .deletehrisabsencesabsenceidrequestbody import (
    DeleteHrisAbsencesAbsenceIDRequestBody,
    DeleteHrisAbsencesAbsenceIDRequestBodyTypedDict,
)
from kombo.types import BaseModel
from kombo.utils import (
    FieldMetadata,
    HeaderMetadata,
    PathParamMetadata,
    RequestMetadata,
)
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class DeleteHrisAbsencesAbsenceIDGlobalsTypedDict(TypedDict):
    integration_id: NotRequired[str]
    r"""ID of the integration you want to interact with."""


class DeleteHrisAbsencesAbsenceIDGlobals(BaseModel):
    integration_id: Annotated[
        Optional[str],
        pydantic.Field(alias="X-Integration-Id"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""ID of the integration you want to interact with."""


class DeleteHrisAbsencesAbsenceIDRequestTypedDict(TypedDict):
    absence_id: str
    r"""The Kombo ID of the absence"""
    body: DeleteHrisAbsencesAbsenceIDRequestBodyTypedDict
    r"""DELETE /hris/absences/:absence_id Request body"""


class DeleteHrisAbsencesAbsenceIDRequest(BaseModel):
    absence_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The Kombo ID of the absence"""

    body: Annotated[
        DeleteHrisAbsencesAbsenceIDRequestBody,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]
    r"""DELETE /hris/absences/:absence_id Request body"""



"""Tests for error handling."""

import pytest
from inline_snapshot import snapshot
from tests.conftest import MockContext
from kombo.errors import (
    KomboAtsError,
    KomboHrisError,
    KomboGeneralError,
    SDKDefaultError,
    ResponseValidationError,
)


class TestErrorHandling:
    """Test error handling behavior."""

    class TestATSEndpoints:
        """Test ATS endpoint error handling."""

        def test_returns_kombo_ats_error_for_platform_rate_limit_errors(self):
            """Test that KomboAtsError is returned for platform rate limit errors."""
            ctx = MockContext()

            ctx.mock_endpoint(
                method="GET",
                path="/v1/ats/jobs",
                response={
                    "status_code": 429,
                    "body": {
                        "status": "error",
                        "error": {
                            "code": "PLATFORM.RATE_LIMIT_EXCEEDED",
                            "title": "Rate limit exceeded",
                            "message": "You have exceeded the rate limit. Please try again later.",
                            "log_url": "https://app.kombo.dev/logs/abc123",
                        },
                    },
                },
            )

            with pytest.raises(KomboAtsError) as exc_info:
                jobs = ctx.kombo.ats.get_jobs()
                if jobs is not None:
                    _ = jobs.next()  # Consume first page

            error = exc_info.value
            assert str(error) == snapshot("You have exceeded the rate limit. Please try again later.")
            assert error.data.error.code == snapshot("PLATFORM.RATE_LIMIT_EXCEEDED")
            assert error.data.error.title == snapshot("Rate limit exceeded")
            assert error.data.error.message == snapshot("You have exceeded the rate limit. Please try again later.")
            assert error.data.error.log_url == snapshot("https://app.kombo.dev/logs/abc123")
            assert error.data.status == snapshot("error")

        def test_returns_kombo_ats_error_for_ats_specific_job_closed_errors(self):
            """Test that KomboAtsError is returned for ATS-specific job closed errors."""
            ctx = MockContext()

            ctx.mock_endpoint(
                method="POST",
                path="/v1/ats/jobs/test-job-id/applications",
                response={
                    "status_code": 400,
                    "body": {
                        "status": "error",
                        "error": {
                            "code": "ATS.JOB_CLOSED",
                            "title": "Job is closed",
                            "message": "Cannot create application for a closed job. The job must be in an open state.",
                            "log_url": "https://app.kombo.dev/logs/ghi789",
                        },
                    },
                },
            )

            with pytest.raises(KomboAtsError) as exc_info:
                ctx.kombo.ats.create_application(
                    job_id="test-job-id",
                    candidate={
                        "first_name": "John",
                        "last_name": "Doe",
                        "email_address": "john.doe@example.com",
                    },
                )

            error = exc_info.value
            assert str(error) == snapshot(
                "Cannot create application for a closed job. The job must be in an open state."
            )
            assert error.data.error.code == snapshot("ATS.JOB_CLOSED")
            assert error.data.error.title == snapshot("Job is closed")
            assert error.data.error.message == snapshot(
                "Cannot create application for a closed job. The job must be in an open state."
            )
            assert error.data.error.log_url == snapshot("https://app.kombo.dev/logs/ghi789")
            assert error.data.status == snapshot("error")

    class TestHRISEndpoints:
        """Test HRIS endpoint error handling."""

        def test_returns_kombo_hris_error_for_integration_permission_errors(self):
            """Test that KomboHrisError is returned for integration permission errors."""
            ctx = MockContext()

            ctx.mock_endpoint(
                method="GET",
                path="/v1/hris/employees",
                response={
                    "status_code": 403,
                    "body": {
                        "status": "error",
                        "error": {
                            "code": "INTEGRATION.PERMISSION_MISSING",
                            "title": "Permission missing",
                            "message": "The integration is missing required permissions to access this resource.",
                            "log_url": "https://app.kombo.dev/logs/hris-def456",
                        },
                    },
                },
            )

            with pytest.raises(KomboHrisError) as exc_info:
                employees = ctx.kombo.hris.get_employees()
                if employees is not None:
                    _ = employees.next()  # Consume first page

            error = exc_info.value
            assert str(error) == snapshot(
                "The integration is missing required permissions to access this resource."
            )
            assert error.data.error.code == snapshot("INTEGRATION.PERMISSION_MISSING")
            assert error.data.error.title == snapshot("Permission missing")
            assert error.data.error.message == snapshot(
                "The integration is missing required permissions to access this resource."
            )
            assert error.data.error.log_url == snapshot("https://app.kombo.dev/logs/hris-def456")
            assert error.data.status == snapshot("error")

    class TestAssessmentEndpoints:
        """Test Assessment endpoint error handling."""

        def test_returns_kombo_ats_error_for_platform_input_validation_errors(self):
            """Test that KomboAtsError is returned for platform input validation errors."""
            ctx = MockContext()

            ctx.mock_endpoint(
                method="GET",
                path="/v1/assessment/orders/open",
                response={
                    "status_code": 400,
                    "body": {
                        "status": "error",
                        "error": {
                            "code": "PLATFORM.INPUT_INVALID",
                            "title": "Input invalid",
                            "message": "The provided input is invalid or malformed.",
                            "log_url": "https://app.kombo.dev/logs/assessment-xyz",
                        },
                    },
                },
            )

            with pytest.raises(KomboAtsError) as exc_info:
                orders = ctx.kombo.assessment.get_open_orders()
                if orders is not None:
                    _ = orders.next()  # Consume first page

            error = exc_info.value
            # Assessment uses KomboAtsError for errors
            assert str(error) == snapshot("The provided input is invalid or malformed.")
            assert error.data.error.code == snapshot("PLATFORM.INPUT_INVALID")
            assert error.data.error.title == snapshot("Input invalid")
            assert error.data.error.message == snapshot("The provided input is invalid or malformed.")
            assert error.data.error.log_url == snapshot("https://app.kombo.dev/logs/assessment-xyz")
            assert error.data.status == snapshot("error")

    class TestGeneralEndpoints:
        """Test General endpoint error handling."""

        def test_returns_kombo_general_error_for_authentication_errors(self):
            """Test that KomboGeneralError is returned for authentication errors."""
            ctx = MockContext()

            ctx.mock_endpoint(
                method="GET",
                path="/v1/check-api-key",
                response={
                    "status_code": 401,
                    "body": {
                        "status": "error",
                        "error": {
                            "code": "PLATFORM.AUTHENTICATION_INVALID",
                            "title": "Authentication invalid",
                            "message": "The provided API key is invalid or expired.",
                            "log_url": "https://app.kombo.dev/logs/general-auth-123",
                        },
                    },
                },
            )

            with pytest.raises(KomboGeneralError) as exc_info:
                ctx.kombo.general.check_api_key()

            error = exc_info.value
            # General endpoints use KomboGeneralError for errors
            assert str(error) == snapshot("The provided API key is invalid or expired.")
            assert error.data.error.code == snapshot("PLATFORM.AUTHENTICATION_INVALID")
            assert error.data.error.title == snapshot("Authentication invalid")
            assert error.data.error.message == snapshot("The provided API key is invalid or expired.")
            assert error.data.error.log_url == snapshot("https://app.kombo.dev/logs/general-auth-123")
            assert error.data.status == snapshot("error")

    class TestUnexpectedResponseFormats:
        """Test handling of unexpected response formats."""

        class TestSDKDefaultErrorForNonJSONResponses:
            """Test SDKDefaultError thrown for non-JSON responses."""

            def test_handles_plain_text_500_error_from_load_balancer(self):
                """Test handling of plain text 500 error from load balancer."""
                ctx = MockContext()

                ctx.mock_endpoint(
                    method="GET",
                    path="/v1/ats/jobs",
                    response={
                        "status_code": 500,
                        "body": "500 Internal Server Error",
                    },
                )

                with pytest.raises(SDKDefaultError) as exc_info:
                    jobs = ctx.kombo.ats.get_jobs()
                    if jobs is not None:
                        _ = jobs.next()  # Consume first page

                error = exc_info.value
                assert str(error) == snapshot(
                    'Unexpected response received: Status 500 Content-Type "". Body: 500 Internal Server Error'
                )

            def test_handles_plain_text_502_bad_gateway_error(self):
                """Test handling of plain text 502 bad gateway error."""
                ctx = MockContext()

                ctx.mock_endpoint(
                    method="GET",
                    path="/v1/hris/employees",
                    response={
                        "status_code": 502,
                        "body": "502 Bad Gateway",
                        "headers": {
                            "Content-Type": "text/plain",
                        },
                    },
                )

                with pytest.raises(SDKDefaultError) as exc_info:
                    employees = ctx.kombo.hris.get_employees()
                    if employees is not None:
                        _ = employees.next()  # Consume first page

                error = exc_info.value
                assert str(error) == snapshot(
                    'Unexpected response received: Status 502 Content-Type text/plain. Body: 502 Bad Gateway'
                )

            def test_handles_html_error_page_from_nginx(self):
                """Test handling of HTML error page from nginx."""
                ctx = MockContext()

                html_error_page = """<!DOCTYPE html>
<html>
<head><title>503 Service Temporarily Unavailable</title></head>
<body>
<center><h1>503 Service Temporarily Unavailable</h1></center>
<hr><center>nginx/1.18.0</center>
</body>
</html>"""

                ctx.mock_endpoint(
                    method="POST",
                    path="/v1/ats/jobs/test-job-id/applications",
                    response={
                        "status_code": 503,
                        "body": html_error_page,
                    },
                )

                with pytest.raises(SDKDefaultError) as exc_info:
                    ctx.kombo.ats.create_application(
                        job_id="test-job-id",
                        candidate={
                            "first_name": "John",
                            "last_name": "Doe",
                            "email_address": "john.doe@example.com",
                        },
                    )

                error = exc_info.value
                assert str(error) == snapshot(
                    """\
Unexpected response received: Status 503 Content-Type "". Body: <!DOCTYPE html>
<html>
<head><title>503 Service Temporarily Unavailable</title></head>
<body>
<center><h1>503 Service Temporarily Unavailable</h1></center>
<hr><center>nginx/1.18.0</center>
</body>
</html>\
"""
                )

            def test_handles_empty_response_body_with_error_status_code(self):
                """Test handling of empty response body with error status code."""
                ctx = MockContext()

                ctx.mock_endpoint(
                    method="GET",
                    path="/v1/check-api-key",
                    response={
                        "status_code": 500,
                        "body": "",
                    },
                )

                with pytest.raises(SDKDefaultError) as exc_info:
                    ctx.kombo.general.check_api_key()

                error = exc_info.value
                assert str(error) == snapshot('Unexpected response received: Status 500 Content-Type "". Body: ""')

            def test_handles_unexpected_content_type_header(self):
                """Test handling of unexpected Content-Type header."""
                ctx = MockContext()

                # Response with unexpected Content-Type
                ctx.mock_endpoint(
                    method="GET",
                    path="/v1/ats/applications",
                    response={
                        "status_code": 500,
                        "body": "Server error occurred",
                        "headers": {
                            "Content-Type": "text/xml",
                        },
                    },
                )

                with pytest.raises(SDKDefaultError) as exc_info:
                    applications = ctx.kombo.ats.get_applications()
                    if applications is not None:
                        _ = applications.next()  # Consume first page

                error = exc_info.value
                assert str(error) == snapshot(
                    'Unexpected response received: Status 500 Content-Type text/xml. Body: Server error occurred'
                )

        def test_handles_unexpected_json_structure_in_error_response(self):
            """Test handling of unexpected JSON structure in error response (ResponseValidationError)."""
            ctx = MockContext()

            # Valid JSON but unexpected structure (not matching Kombo error format)
            unexpected_json = {
                "errorCode": "500",
                "errorMessage": "Internal server error",
                "timestamp": "2024-01-01T00:00:00Z",
            }

            ctx.mock_endpoint(
                method="GET",
                path="/v1/ats/jobs",
                response={
                    "status_code": 500,
                    "body": unexpected_json,
                },
            )

            with pytest.raises(ResponseValidationError) as exc_info:
                jobs = ctx.kombo.ats.get_jobs()
                if jobs is not None:
                    _ = jobs.next()  # Consume first page

            error = exc_info.value
            # Valid JSON but unexpected structure triggers ResponseValidationError
            assert "Response validation failed" in str(error)


